# doc
# https://github.com/hgollnick/whisper_mic_dual_language/tree/main

import whisper
import queue
import threading
import torch
import numpy as np
import speech_recognition as sr
import os


def main():
    audio_queue = queue.Queue()
    threading.Thread(target=record, args={audio_queue}).start()
    threading.Thread(target=transcribe, args={audio_queue}).start()


def init_whisper():
    # choisir le modèle ici
    model = whisper.load_model("base")
    # comment faire pour download les models ??
    # model_root = os.path.expanduser("~/.cache/whisper")
    # model = whisper.load_model("medium", download_root=model_root)
    return model


def init_recognizer():
    r = sr.Recognizer()
    r.energy_threshold = 300
    r.pause_threshold = 0.8
    r.dynamic_energy_threshold = False
    return r


def record(audio_queue):
    r = init_recognizer()
    print("Parlez!")
    while True:
        with sr.Microphone(sample_rate=16000) as source:
        # changer l'input microphone : with sr.Microphone(device_index=0) as source:
            audio = r.listen(source)
            torch_audio = torch.from_numpy(np.frombuffer(audio.get_raw_data(), np.int16).flatten().astype(np.float32) / 32768.0)
            audio_queue.put_nowait(torch_audio)
            

def transcribe(audio_queue):
    while True:
        audio = audio_queue.get()
        model = init_whisper()
        # fp16=False permet d'enlever l'erreur lié au CPU
        # language="french" permet de définir le langage de base
        transcried_audio = model.transcribe(audio, fp16=False, language="fr")
        translated_audio = model.transcribe(audio, task="translate", fp16=False) 

        print("\n")
        print("Language: " + transcried_audio["language"])
        print("Input: " + transcried_audio["text"])
        print("Translation: " + translated_audio["text"])

        with open("transcried_audio.txt","a") as file:
            file.write("<p>" + transcried_audio["text"] + "</p>\n")

main()

